$(document).ready(() => {
    let Sound = function (src) {
        this.sound = document.createElement("audio");
        this.sound.src = src;
        this.sound.setAttribute("preload", "auto");
        this.sound.setAttribute("controls", "none");
        this.sound.style.display = "none";
        document.body.appendChild(this.sound);
        this.play = function(){
            this.sound.play();
        }
        this.stop = function(){
            this.sound.pause();
        }
    }

    let Regnestykke = function (art, tal1, tal2, includeNegativeNumbers) {
        this.art = art;
        this.tal1 = Math.floor(Math.random() * (tal2 - 1)); // tal 1 skal minimum være 1 lavere end tal2! :/
        this.tal2 = Math.floor(Math.random() * (tal2 - this.tal1)) + this.tal1;
        switch (art) {
        case '+' :  this.answer = this.tal1 + this.tal2;
                    break;
        case '-' :  if (!includeNegativeNumbers) { // swap numbers to avoid minus numbers
                        let tmp = this.tal1; this.tal1 = this.tal2; this.tal2 = tmp;
                    }
                    this.answer = this.tal1 - this.tal2;
                    break;
        case '*' :  this.answer = this.tal1 * this.tal2;
                    break;
        case '/' :  let tmp1 = this.tal1; this.tal1 = this.tal2; this.tal2 = tmp1;
                    if (this.tal2 === 0)
                        this.tal2++;
                    while(this.tal1 % this.tal2 !== 0)
                        if (Math.random() > 0.5) this.tal1--; else this.tal2--;
                    this.answer = this.tal1 / this.tal2;
                    break;
        case '%' :  let tmp2 = this.tal1; this.tal1 = this.tal2; this.tal2 = tmp2;
                    this.answer = this.tal1 % this.tal2;
                    break;
        default :  this.answer = this.tal1 + this.tal2;
                    break;
        }
        this.element = $(`<div class="regnestykke row"><div class="col-12"><div class="stykke"><div class="tal1">${this.tal1}</div><div class="regneart">${this.art}</div><div class="tal2">${this.tal2}</div></div><div class="equals">=</div><div class="answer"><input type="text"> <button class="svar btn btn-info btn-xs"> Svar </button></div><div class="result"></result></div></div>`);
        this.input = $('input', this.element);
        this.hasNewGuess = false;
    };

    Regnestykke.prototype = {
        test: function (guess) {
            if (guess.length === 0)
                throw 'Intet indhold';
            this.hasNewGuess = false;
            if (guess > this.answer)
                return 1;
            if (guess < this.answer)
                return -1;
            return 0;
        },
        respond: function (response) {
            let resultElem = this.element.find('.result');
            if (response === 0) {
                resultElem.removeClass('fail').addClass('success');
                resultElem.html('Rigtigt! <img src="statics/success.gif">');
                if (hf.everythingIsSolved()) {
                    hf.success();
                    hf.flashWon();
                } else {
                    hf.right();
                }
            } else {
                resultElem.removeClass('success').addClass('fail');
                hf.wrong();
                if (response < 0) {
                    resultElem.html('Højere   (prøv igen) <img src="statics/wrong.png">');
                } else {
                    resultElem.html('Lavere   (prøv igen) <img src="statics/wrong.png">');
                }
            }
        },
        isSolved: function () {
            let guess = this.input[0].value;
            if (guess.length < 1)
                return false;
            return parseInt(guess, 10) === this.answer;
        }
    }
    
    let Hf = function (regnearter, min, max, antalStykker) {
        this.regnearter = regnearter;
        this.possibleRegnearter = '+-*/%';
        this.min = min || 0;
        this.max = max || 20;
        this.antalStykker = antalStykker || 5;
        this.opgaver = null;
    };

    Hf.prototype = {
        getRegnestykker: function (antal, arter, min, max) {
            let stykker = [];
            for (let i=0; i < antal; i++) {
                let art;
                if (arter.length === 1) {
                    art = arter;
                } else 
                    art = arter.charAt(Math.floor(Math.random() * arter.length));
                stykker.push(new Regnestykke(art, min, max));
            }
            return stykker;
        },
        wrongSound: new Sound("statics/wrong.mp3"),
        rightSound: new Sound("statics/right.wav"),
        successSound: new Sound("statics/finalFanfare.mp3"),
        right: function () {
            this.rightSound.play();
        },
        wrong: function () {
            this.wrongSound.play();
        },
        success: function () {
            this.successSound.play();
        },
        refresh: function () {
            $('#regnestykker').html('');
            this.opgaver = hf.getRegnestykker(this.antalStykker,this.regnearter, this.min, this.max);
            this.opgaver.forEach(op => {
                $('button', op.element).click(e => {
                    try{
                        op.respond(op.test(op.input[0].value));
                    } catch (e) {}
                });
                $('input', op.element).blur(e => {
                    if (op.hasNewGuess)
                        try {
                            op.respond(op.test(op.input[0].value));
                        } catch (e) {}
                });
                $('input', op.element).keydown(e => {
                    if (e.originalEvent.keyCode === 13) {
                        e.preventDefault();
                        if (op.hasNewGuess)
                            try {
                                op.respond(op.test(op.input[0].value));
                            } catch (e) {}
                        hf.focusNextField(op);
                    } else {
                        if (e.originalEvent.keyCode > 47 && e.originalEvent.keyCode < 57 || e.originalEvent.keyCode === 8 || e.originalEvent.keyCode === 46) // numbers or backspace and del
                            op.hasNewGuess = true;
                            return true;
                    }
                });
                $('#regnestykker').append(op.element);
            });
        },
        focusNextField: function(opgave) {
            let opgaveIndex = 0;
            while(opgave !== this.opgaver[opgaveIndex])
                opgaveIndex++;
            if (opgaveIndex === this.opgaver.length - 1) {
                $('#flereOpgaver').focus();
            } else {
                this.opgaver[opgaveIndex + 1].input.focus();
            }
        },
        everythingIsSolved: function() {
            let everythingIsSolved = true;
            this.opgaver.forEach((op) => {
                everythingIsSolved = everythingIsSolved && op.isSolved();
            });
            return everythingIsSolved;
        },
        flashWon: function() {
            $('#final').addClass('on');
            setTimeout(() => { $('#final').addClass('visible'); }, 20);
            setTimeout(() => { $('#final').addClass('hideIt'); }, 4000);
            setTimeout(() => { $('#final').removeClass('on visible hideIt'); }, 6500);
        },
        populateControlpanel: function() {
            $('#control-plus')[0].checked = this.regnearter.indexOf('+') > -1;
            $('#control-minus')[0].checked = this.regnearter.indexOf('-') > -1;
            $('#control-gange')[0].checked = this.regnearter.indexOf('*') > -1;
            $('#control-dividere')[0].checked = this.regnearter.indexOf('/') > -1;
            $('#control-modulus')[0].checked = this.regnearter.indexOf('%') > -1;
            $('#control-min')[0].value = this.min;
            $('#control-max')[0].value = this.max;
            $('#control-antalOpgaver')[0].value = this.antalStykker;
        },
        updateControlValues: function() {
            let regnearter = '';
            if ($('#control-plus')[0].checked) regnearter += '+';
            if ($('#control-minus')[0].checked) regnearter += '-';
            if ($('#control-gange')[0].checked) regnearter += '*';
            if ($('#control-dividere')[0].checked) regnearter += '/';
            if ($('#control-modulus')[0].checked) regnearter += '%';
            hf.regnearter = regnearter;
            hf.min = parseInt($('#control-min')[0].value, 10);
            hf.max = parseInt($('#control-max')[0].value, 10);
            hf.antalStykker = parseInt($('#control-antalOpgaver')[0].value, 10);
        }
    };

    $('#controlboard #controlToggler').click((e) => {
        $('#controlboard').toggleClass('in');
    });
    
    window.hf = new Hf('+-', 0, 20, 5);
    hf.populateControlpanel();
    hf.refresh();
});
